from openpyxl import load_workbook
from openpyxl.utils import get_column_letter
from typing import List, Dict


class Notebook:

    def __init__(self, file: str, sheet: str = "active"):
        self.file = file
        self.sheet = sheet
        self.book = load_workbook(file).active \
            if sheet == "active" \
            else load_workbook(file)[sheet]

    def data_rows(self) -> List[str]:
        return [
            cell.value for row in self.book.iter_rows()
            for cell in row
        ]

    def data_cols(self) -> List[str]:
        return [
            cell.value for column in self.book.iter_cols()
            for cell in column
        ]

    def list_rows(self, start: int = 1, end: int = None) -> List[List]:
        return [
            [cell.value for cell in row]
            for row in self.book.iter_rows(min_row=start, max_row=end)
        ]

    def dict_rows(self, start: int = 1, end: int = None) -> Dict[int, List]:
        cells_value_dict = {}
        for index, row in enumerate(
                self.list_rows(start=start, end=end)
        ):
            cells_value_dict.setdefault(start + index, [])
            for value in row:
                cells_value_dict[start + index].append(value)
        return cells_value_dict

    def list_cols(self, start: int = 1, end: int = None) -> List[List]:
        return [
            [cell.value for cell in column]
            for column in self.book.iter_cols(min_row=start, max_row=end)
        ]

    def dict_cols(self, start: int = 1, end: int = None) -> Dict[int, List]:
        cells_value_dict = {}
        for index, col in enumerate(
                self.list_cols(start=start, end=end)
        ):
            cells_value_dict.setdefault(index + 1, [])
            for value in col:
                cells_value_dict[index + 1].append(value)
        return cells_value_dict

    def headers(self, row: int = 1) -> List[str]:
        return [
            cell.value for row in self.book.iter_rows(
                min_row=row, max_row=row
            ) for cell in row
        ]

    def value_rows(self, value: str) -> List[List[str]]:
        return [
            row for row in self.list_rows() if value in row
        ]

    def value_cols(self, value: str) -> List[List[str]]:
        return [
            col for col in self.list_cols() if value in col
        ]

    def data_convert(self, headers_row: int = 1) -> Dict[str, List]:
        convert_book = self.book
        headers = self.headers()
        cells_value_dict = {}
        for header in headers:
            if header is not None:
                cells_value_dict.setdefault(header, [])
            else:
                continue
            for row in [
                [cell.value for cell in row]
                for row in convert_book.iter_rows(
                    min_row=headers_row + 1
                )
            ]:
                cells_value_dict[header].append(
                    row[headers.index(header)]
                )
        return cells_value_dict

    def header_values(self, header: str) -> List[str]:
        header_values = []
        for index_row, header_row in enumerate(
                self.list_rows()
        ):
            if header in header_row:
                for index, cell_header in enumerate(header_row):
                    if cell_header == header:
                        for row in self.list_rows(
                                start=index_row + 2
                        ):
                            header_values.append(row[index])
        return header_values

    def find(self, value: str) -> bool:
        for cell_value in self.data_rows():
            if cell_value == value:
                return True
        return False

    def value_position(self, value: str = "") -> str:
        for row in self.book.iter_rows():
            for cell in row:
                if cell.value == value:
                    return f"{get_column_letter(cell.column)}{cell.row}"

    def value_coordinates(self, value: str = "") -> tuple:
        for row in self.book.iter_rows():
            for cell in row:
                if cell.value == value:
                    return cell.row, cell.column



import os


class TestFile:

    @staticmethod
    def test_xlsx_file(test_file):
        for file_in_dir in os.listdir():
            if file_in_dir.endswith('.xlsx'):
                assert file_in_dir == test_file

    def test_tests_sheet_exist(self, tests_book):
        assert tests_book.headers() == [
            'ID', 'Status', 'Test Name', 'Test Description',
        ]

    def test_rows_and_cols_sheet_exist(self, rows_and_cols_book):
        assert rows_and_cols_book.headers() == [
            'COLUMN A', 'COLUMN B', 'COLUMN C', 'COLUMN D', 'COLUMN E', 'COLUMN F'
        ]



import pytest
from xlsxopera import Notebook

file = "test.xlsx"


@pytest.fixture()
def test_file():
    return file


@pytest.fixture()
def tests_book():
    return Notebook(file, "tests")


@pytest.fixture()
def rows_and_cols_book():
    return Notebook("test.xlsx", "rows and cols")


@pytest.fixture()
def data_convert_book():
    return Notebook("test.xlsx", "data convert")




class TestRowsAndCols:

    def test_get_cols_with_value(self, rows_and_cols_book):
        assert rows_and_cols_book.value_cols("John Smith") == [
            [
                'COLUMN E',
                'value_1_column_E',
                'value_2_column_E',
                'value_3_column_E',
                'value_1_column_E_align_center',
                'value_1_column_E_center_vertically',
                'John Smith'
                ]
        ]

    def test_get_rows_with_value(self, rows_and_cols_book):
        assert rows_and_cols_book.value_rows("value_3_column_F") == [
            [
                'value_3_column_A',
                'value_3_column_B',
                'value_3_column_C',
                'value_3_column_D',
                'value_3_column_E',
                'value_3_column_F'
                ]
        ]

    def test_get_cell_coordinates_by_value(self, rows_and_cols_book):
        assert rows_and_cols_book.value_coordinates(
            "value_1_column_F"
        ) == (2, 6)
        assert rows_and_cols_book.value_coordinates(
            "value_3_column_A"
        ) == (4, 1)

    def test_get_cell_position_by_value(self, rows_and_cols_book):
        assert rows_and_cols_book.value_position(
            "value_3_column_C"
        ) == "C4"
        assert rows_and_cols_book.value_position(
            "value_2_column_E"
        ) == "E3"

    def test_find_cell_by_value(self, rows_and_cols_book):
        assert True if rows_and_cols_book.find("value_2_column_B") \
                       and rows_and_cols_book.find("Value_that_not_exist") is False else False

    def test_get_values_below_the_header(self, rows_and_cols_book):
        assert rows_and_cols_book.header_values("COLUMN D") == [
            'value_1_column_D', 'value_2_column_D', 'value_3_column_D',
            'value_1_column_D_align_center', 'value_1_column_D_center_vertically',
            'environment'
        ]
        assert rows_and_cols_book.header_values("value_1_column_E_align_center") == [
            'value_1_column_E_center_vertically', 'John Smith'
        ]

    def test_get_headers(self, rows_and_cols_book):
        assert rows_and_cols_book.headers() == [
            "COLUMN A", "COLUMN B", "COLUMN C", "COLUMN D", "COLUMN E", "COLUMN F"
        ]

    def test_get_all_data_converted_to_dict(self, data_convert_book):
        assert data_convert_book.data_convert() == {
            "COLUMN A":
                [
                    'value_1_column_A', 'value_2_column_A', 'value_3_column_A'
                ],
            "COLUMN B":
                [
                    'value_1_column_B', 'value_2_column_B', 'value_3_column_B'
                ],
        }

    def test_get_cols_2_3_in_dict(self, rows_and_cols_book):
        """
        Get dict of cols from 2 to 3 and compare.
        """
        assert rows_and_cols_book.dict_cols(start=3, end=4) == {
            1:
                [
                    'value_2_column_A', 'value_3_column_A'
                ],
            2:
                [
                    'value_2_column_B', 'value_3_column_B'
                ],
            3:
                [
                    'value_2_column_C', 'value_3_column_C'
                ],
            4:
                [
                    'value_2_column_D', 'value_3_column_D'
                ],
            5:
                [
                    'value_2_column_E', 'value_3_column_E'
                ],
            6:
                [
                    'value_2_column_F', 'value_3_column_F'
                ],
        }

    def test_get_cols_2_3_4_in_list(self, rows_and_cols_book):
        """
        Get lists of cols from 2 to 4 and compare.
        """
        assert rows_and_cols_book.list_cols(start=2, end=4) == [
            [
                'value_1_column_A', 'value_2_column_A', 'value_3_column_A'
            ],
            [
                'value_1_column_B', 'value_2_column_B', 'value_3_column_B'
            ],
            [
                'value_1_column_C', 'value_2_column_C', 'value_3_column_C'
            ],
            [
                'value_1_column_D', 'value_2_column_D', 'value_3_column_D'
            ],
            [
                'value_1_column_E', 'value_2_column_E', 'value_3_column_E'
            ],
            [
                'value_1_column_F', 'value_2_column_F', 'value_3_column_F'
            ]
        ]

    def test_get_rows_2_3_in_dict(self, rows_and_cols_book):
        """
        Get dict of rows from 2 to 4.
        """
        assert rows_and_cols_book.dict_rows(start=3, end=4) == {
            3:
            [
                'value_2_column_A',
                'value_2_column_B',
                'value_2_column_C',
                'value_2_column_D',
                'value_2_column_E',
                'value_2_column_F'
            ],
            4:
            [
                'value_3_column_A',
                'value_3_column_B',
                'value_3_column_C',
                'value_3_column_D',
                'value_3_column_E',
                'value_3_column_F'
            ]
        }

    def test_get_rows_2_3_4_in_list(self, rows_and_cols_book):
        """
        Get lists rows from 2 to 4.
        """
        assert rows_and_cols_book.list_rows(start=2, end=4) == [
            [
                'value_1_column_A',
                'value_1_column_B',
                'value_1_column_C',
                'value_1_column_D',
                'value_1_column_E',
                'value_1_column_F'
            ],
            [
                'value_2_column_A',
                'value_2_column_B',
                'value_2_column_C',
                'value_2_column_D',
                'value_2_column_E',
                'value_2_column_F'
            ],
            [
                'value_3_column_A',
                'value_3_column_B',
                'value_3_column_C',
                'value_3_column_D',
                'value_3_column_E',
                'value_3_column_F'
            ]
        ]

    def test_get_data_in_list_by_columns(self, rows_and_cols_book):
        """
        Get first 2 values and last 2 values in sheet. Listing by columns get first rows from position A1 and A2.
        Same logic for last 2 values.
        """
        assert True if rows_and_cols_book.data_cols()[0:2] == [
            'COLUMN A', 'value_1_column_A'
        ] and rows_and_cols_book.data_cols()[-2::] == [
            'value_1_column_F_center_vertically', None
        ] else False

    def test_get_data_in_list_by_rows(self, rows_and_cols_book):
        """
        Get first 2 values and last 2 values in sheet. Listing by rows get first rows from position A1 and B1.
        Same logic for last 2 values.
        """
        assert True if rows_and_cols_book.data_rows()[0:2] == [
            'COLUMN A', 'COLUMN B'
        ] and rows_and_cols_book.data_rows()[-2::] == [
            'John Smith', None
        ] else False



from setuptools import setup, find_packages


setup(
    name="xlsxopera",
    version="1.1",
    author="voidayan",
    author_email="jan.emil.wojda@gmail.com",
    url='https://github.com/voidayan/xlsxopera',
    description="Work on .xlsx files.",
    long_description_content_type="text/markdown",
    long_description=open('README.md').read(),
    packages=find_packages(),
    install_requires=['openpyxl'],
    keywords=['python', 'xlsx', 'xls', 'excel', 'microsoft excel', 'cell'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
